import time
from ._result import Result as R
from ._scenarios import Scenarios as S
from ._variables import Variables as V


class Play():
    """
    Invidual step in a playbook, does not have direct knowledge of the
    scenario it relates to, but instead a unique reference to a name.
    Manages parsing of the response and time to the next step
    """

    NAME: str
    WAIT: int
    ASSERT: str
    STEP: int
    RETURN_BODY: bool
    result: R
    scenario: S

    def __init__(self, return_body=False) -> None:
        self.RETURN_BODY = return_body
        pass

    def run(self, scenarios: S, variables: V) -> R:
        ret_val = scenarios.run_scenario(
            self.NAME, variables, return_body=self.RETURN_BODY)
        time.sleep(self.WAIT)
        return ret_val






from itertools import groupby
from ._utils import utils
from ._play import Play as P
from ._result import Result as R
from ._scenarios import Scenarios as S
from ._variables import Variables as V


class Playbook():
    """Parent playbook class, manages each of the steps provided
    """

    plays: list[P]
    u: utils = utils()

    def __init__(self, playbook_file: str = None) -> None:
        self._parse_playbook_file(playbook_file)
        self.step = None

    def _parse_playbook_file(self, file: str):
        self.plays = self._parse_play_list(file)

    def _parse_play_list(self, file:str):
        playbook_blocks = self._split_playbook_file(file)
        return list(map(self._parse_block, playbook_blocks))


    def _split_playbook_file(self, file: str) -> list:
        raw_data = self.u.load_file(file)
        split_scenarios = [list(g) for k, g in groupby(
            raw_data, key=lambda x: x != "----") if k]
        return split_scenarios

    def _parse_block(self, entry: list):
        p = P()
        props = self.u.set_properties(entry)
        if props.get('NAME', None) is None:
            raise ValueError("Play is missing NAME")
        p.NAME = props.get('NAME')
        p.WAIT = int(props.get('WAIT', 30))
        p.STEP = props.get('STEP')
        p.ASSERT = props.get('ASSERT')
        p.NEXT_STEP = props.get('NEXT_STEP',None)

        return p

    def run(self, scenarios: S, variables: V) -> list[R]:
        ret_val = []
        for p in self.plays:
            ret_val.append(p.run(scenarios, variables))
        return ret_val



"""
Module providing RESTClient class
"""
import os

from pathlib import Path
from ._scenarios import Scenarios as S
from ._variables import Variables as V
from ._playbook import Playbook as P
from ._result import Result as R


class RESTPlaybook():
    """Top level class for running playbooks, manages the orchestration
    of each file
    """
    app_name: str = 'rest_playbook_micro'
    config_dir: str = os.path.join(
        str(Path.home()),
        ".config/",
        app_name
    )

    scenario_file: str = ''
    var_file: str = ''
    playbook_file: str = ''

    scenarios: S
    variables: V
    playbook: P

    def __init__(self, p_file: str, s_file: str, v_file: str) -> None:
        self.set_scenario_file(s_file)
        self.set_var_file(v_file)
        self.set_playbook_file(p_file)

    def set_playbook_file(self, playbook_file: str) -> None:
        self.playbook_file = playbook_file
        self.generate_playbook(self.playbook_file)

    def generate_playbook(self, playbook_file: str) -> None:
        self.playbook = P(playbook_file)

    def set_scenario_file(self, scenario_file: str) -> None:
        self.scenario_file = scenario_file
        self.generate_scenarios(self.scenario_file)

    def set_var_file(self, var_file: str) -> None:
        self.var_file = var_file
        self.generate_variables(self.var_file)

    def generate_variables(self, var_file: str) -> None:
        self.variables = V(var_file)

    def generate_scenarios(self, scenario_file: str) -> None:
        self.scenarios = S(scenario_file)

    def run_scenario(self, scenario_name: str) -> R:
        return self.scenarios.run_scenario(
            scenario_name, self.variables
        )
    
    def run_playbook(self) -> list[R]:
        return self.playbook.run(self.scenarios, self.variables)

    def print(self) -> None:
        self.scenarios.print()
        self.variables.print()
